@@ -44,7 +44,7 @@ stop() ->
 
 
 all() ->
-    lists:sort(gen_server:call(?MODULE, all)).
+    lists:sort(gen_server:call(?MODULE, all, infinity)).
 
 
 get(Section) when is_binary(Section) ->
@@ -111,7 +111,7 @@ terminate(_Reason, _State) ->
 handle_call(all, _From, Config) ->
     Resp = lists:sort((ets:tab2list(?MODULE))),
     {reply, Resp, Config};
-handle_call({set, Sec, Key, Val, Persist}, _From, Config) ->
+handle_call({set, Sec, Key, Val, Persist}, From, Config) ->
     true = ets:insert(?MODULE, {{Sec, Key}, Val}),
     case {Persist, Config#config.write_filename} of
         {true, undefined} ->
@@ -121,9 +121,12 @@ handle_call({set, Sec, Key, Val, Persist}, _From, Config) ->
         _ ->
             ok
     end,
-    [catch F(Sec, Key, Val, Persist) || {_Pid, F} <- Config#config.notify_funs],
-    {reply, ok, Config};
-handle_call({delete, Sec, Key, Persist}, _From, Config) ->
+    spawn_link(fun() ->
+        [catch F(Sec, Key, Val, Persist) || {_Pid, F} <- Config#config.notify_funs],
+            gen_server:reply(From, ok)
+    end),
+    {noreply, Config};
+handle_call({delete, Sec, Key, Persist}, From, Config) ->
     true = ets:delete(?MODULE, {Sec,Key}),
     case {Persist, Config#config.write_filename} of
         {true, undefined} ->
@@ -133,8 +136,11 @@ handle_call({delete, Sec, Key, Persist}, _From, Config) ->
         _ ->
             ok
     end,
-    [catch F(Sec, Key, deleted, Persist) || {_Pid, F} <- Config#config.notify_funs],
-    {reply, ok, Config};
+    spawn_link(fun() ->
+        [catch F(Sec, Key, deleted, Persist) || {_Pid, F} <- Config#config.notify_funs],
+            gen_server:reply(From, ok)
+    end),
+    {noreply, Config};
 handle_call({register, Fun, Pid}, _From, #config{notify_funs=PidFuns}=Config) ->
     erlang:monitor(process, Pid),
     % convert 1 and 2 arity to 3 arity
@@ -225,7 +231,7 @@ parse_ini_file(IniFile) ->
                     [[]] ->
                         % empty line means delete this key
                         ets:delete(?MODULE, {AccSectionName, ValueName}),
-                        {AccSectionName, AccValues};                        
+                        {AccSectionName, AccValues};
                     [LineValue | _Rest] ->
                         {AccSectionName,
                             [{{AccSectionName, ValueName}, LineValue} | AccValues]}
