@@ -16,8 +16,8 @@
 -export([handle_view_req/3,handle_temp_view_req/2]).
 
 -export([get_stale_type/1, get_reduce_type/1, parse_view_params/3]).
--export([make_view_fold_fun/6, finish_view_fold/4, view_row_obj/3]).
--export([view_group_etag/2, view_group_etag/3, make_reduce_fold_funs/5]).
+-export([make_view_fold_fun/7, finish_view_fold/4, finish_view_fold/5, view_row_obj/3]).
+-export([view_group_etag/2, view_group_etag/3, make_reduce_fold_funs/6]).
 -export([design_doc_view/5, parse_bool_param/1, doc_member/2]).
 -export([make_key_options/1, load_view/4]).
 
@@ -26,6 +26,8 @@
     start_json_response/2, start_json_response/3, end_json_response/1,
     send_chunked_error/2]).
 
+-import(couch_db,[get_update_seq/1]).
+
 design_doc_view(Req, Db, DName, ViewName, Keys) ->
     DesignId = <<"_design/", DName/binary>>,
     Stale = get_stale_type(Req),
@@ -59,8 +61,9 @@ handle_view_req(#httpd{method='GET',
 
 handle_view_req(#httpd{method='POST',
         path_parts=[_, _, DName, _, ViewName]}=Req, Db, _DDoc) ->
+    couch_httpd:validate_ctype(Req, "application/json"),
     {Fields} = couch_httpd:json_body_obj(Req),
-    case proplists:get_value(<<"keys">>, Fields, nil) of
+    case couch_util:get_value(<<"keys">>, Fields, nil) of
     nil ->
         Fmt = "POST to view ~p/~p in database ~p with no keys member.",
         ?LOG_DEBUG(Fmt, [DName, ViewName, Db]),
@@ -75,14 +78,16 @@ handle_view_req(Req, _Db, _DDoc) ->
     send_method_not_allowed(Req, "GET,POST,HEAD").
 
 handle_temp_view_req(#httpd{method='POST'}=Req, Db) ->
+    couch_httpd:validate_ctype(Req, "application/json"),
+    ok = couch_db:check_is_admin(Db),
     couch_stats_collector:increment({httpd, temporary_view_reads}),
     {Props} = couch_httpd:json_body_obj(Req),
-    Language = proplists:get_value(<<"language">>, Props, <<"javascript">>),
-    {DesignOptions} = proplists:get_value(<<"options">>, Props, {[]}),
-    MapSrc = proplists:get_value(<<"map">>, Props),
-    Keys = proplists:get_value(<<"keys">>, Props, nil),
+    Language = couch_util:get_value(<<"language">>, Props, <<"javascript">>),
+    {DesignOptions} = couch_util:get_value(<<"options">>, Props, {[]}),
+    MapSrc = couch_util:get_value(<<"map">>, Props),
+    Keys = couch_util:get_value(<<"keys">>, Props, nil),
     Reduce = get_reduce_type(Req),
-    case proplists:get_value(<<"reduce">>, Props, null) of
+    case couch_util:get_value(<<"reduce">>, Props, null) of
     null ->
         QueryArgs = parse_view_params(Req, Keys, map),
         {ok, View, Group} = couch_view:get_temp_map_view(Db, Language,
@@ -111,11 +116,11 @@ output_map_view(Req, View, Group, Db, QueryArgs, nil) ->
     CurrentEtag = view_group_etag(Group, Db),
     couch_httpd:etag_respond(Req, CurrentEtag, fun() ->
         {ok, RowCount} = couch_view:get_row_count(View),
-        FoldlFun = make_view_fold_fun(Req, QueryArgs, CurrentEtag, Db, RowCount, #view_fold_helper_funs{reduce_count=fun couch_view:reduce_to_count/1}),
+        FoldlFun = make_view_fold_fun(Req, QueryArgs, CurrentEtag, Db, Group#group.current_seq, RowCount, #view_fold_helper_funs{reduce_count=fun couch_view:reduce_to_count/1}),
         FoldAccInit = {Limit, SkipCount, undefined, []},
-        {ok, LastReduce, FoldResult} = couch_view:fold(View, 
+        {ok, LastReduce, FoldResult} = couch_view:fold(View,
                 FoldlFun, FoldAccInit, make_key_options(QueryArgs)),
-        finish_view_fold(Req, RowCount, 
+        finish_view_fold(Req, RowCount,
                 couch_view:reduce_to_count(LastReduce), FoldResult)
     end);
 
@@ -128,19 +133,19 @@ output_map_view(Req, View, Group, Db, QueryArgs, Keys) ->
     couch_httpd:etag_respond(Req, CurrentEtag, fun() ->
         {ok, RowCount} = couch_view:get_row_count(View),
         FoldAccInit = {Limit, SkipCount, undefined, []},
-        {LastReduce, FoldResult} = lists:foldl(
-            fun(Key, {_, FoldAcc}) ->
-                FoldlFun = make_view_fold_fun(Req,
-                    QueryArgs#view_query_args{
-                    }, CurrentEtag, Db, RowCount,
+        {LastReduce, FoldResult} = lists:foldl(fun(Key, {_, FoldAcc}) ->
+            FoldlFun = make_view_fold_fun(Req, QueryArgs#view_query_args{},
+                    CurrentEtag, Db, Group#group.current_seq, RowCount,
                     #view_fold_helper_funs{
                         reduce_count = fun couch_view:reduce_to_count/1
                     }),
-                {ok, LastReduce, FoldResult} = couch_view:fold(View, FoldlFun, FoldAcc, 
-                    make_key_options(QueryArgs#view_query_args{start_key=Key, end_key=Key})),
-                {LastReduce, FoldResult}
-            end, {{[],[]}, FoldAccInit}, Keys),
-        finish_view_fold(Req, RowCount, couch_view:reduce_to_count(LastReduce), FoldResult)
+            {ok, LastReduce, FoldResult} = couch_view:fold(View, FoldlFun,
+                    FoldAcc, make_key_options(
+                         QueryArgs#view_query_args{start_key=Key, end_key=Key})),
+            {LastReduce, FoldResult}
+        end, {{[],[]}, FoldAccInit}, Keys),
+        finish_view_fold(Req, RowCount, couch_view:reduce_to_count(LastReduce),
+                FoldResult, [{update_seq,Group#group.current_seq}])
     end).
 
 output_reduce_view(Req, Db, View, Group, QueryArgs, nil) ->
@@ -151,7 +156,9 @@ output_reduce_view(Req, Db, View, Group, QueryArgs, nil) ->
     } = QueryArgs,
     CurrentEtag = view_group_etag(Group, Db),
     couch_httpd:etag_respond(Req, CurrentEtag, fun() ->
-        {ok, GroupRowsFun, RespFun} = make_reduce_fold_funs(Req, GroupLevel, QueryArgs, CurrentEtag, #reduce_fold_helper_funs{}),
+        {ok, GroupRowsFun, RespFun} = make_reduce_fold_funs(Req, GroupLevel,
+                QueryArgs, CurrentEtag, Group#group.current_seq,
+                #reduce_fold_helper_funs{}),
         FoldAccInit = {Limit, Skip, undefined, []},
         {ok, {_, _, Resp, _}} = couch_view:fold_reduce(View,
                 RespFun, FoldAccInit, [{key_group_fun, GroupRowsFun} |
@@ -167,10 +174,13 @@ output_reduce_view(Req, Db, View, Group, QueryArgs, Keys) ->
     } = QueryArgs,
     CurrentEtag = view_group_etag(Group, Db, Keys),
     couch_httpd:etag_respond(Req, CurrentEtag, fun() ->
-        {ok, GroupRowsFun, RespFun} = make_reduce_fold_funs(Req, GroupLevel, QueryArgs, CurrentEtag, #reduce_fold_helper_funs{}),
+        {ok, GroupRowsFun, RespFun} = make_reduce_fold_funs(Req, GroupLevel,
+                QueryArgs, CurrentEtag, Group#group.current_seq,
+                #reduce_fold_helper_funs{}),
         {Resp, _RedAcc3} = lists:foldl(
             fun(Key, {Resp, RedAcc}) ->
-                % run the reduce once for each key in keys, with limit etc reapplied for each key
+                % run the reduce once for each key in keys, with limit etc
+                % reapplied for each key
                 FoldAccInit = {Limit, Skip, Resp, RedAcc},
                 {_, {_, _, Resp2, RedAcc2}} = couch_view:fold_reduce(View,
                         RespFun, FoldAccInit, [{key_group_fun, GroupRowsFun} |
@@ -180,7 +190,7 @@ output_reduce_view(Req, Db, View, Group, QueryArgs, Keys) ->
                 {Resp2, RedAcc2}
             end,
         {undefined, []}, Keys), % Start with no comma
-        finish_reduce_fold(Req, Resp)
+        finish_reduce_fold(Req, Resp, [{update_seq,Group#group.current_seq}])
     end).
 
 reverse_key_default(?MIN_STR) -> ?MAX_STR;
@@ -188,28 +198,28 @@ reverse_key_default(?MAX_STR) -> ?MIN_STR;
 reverse_key_default(Key) -> Key.
 
 get_stale_type(Req) ->
-    list_to_atom(couch_httpd:qs_value(Req, "stale", "nil")).
+    list_to_existing_atom(couch_httpd:qs_value(Req, "stale", "nil")).
 
 get_reduce_type(Req) ->
-    list_to_atom(couch_httpd:qs_value(Req, "reduce", "true")).
+    list_to_existing_atom(couch_httpd:qs_value(Req, "reduce", "true")).
 
 load_view(Req, Db, {ViewDesignId, ViewName}, Keys) ->
-    Stale = couch_httpd_view:get_stale_type(Req),
-    Reduce = couch_httpd_view:get_reduce_type(Req),
+    Stale = get_stale_type(Req),
+    Reduce = get_reduce_type(Req),
     case couch_view:get_map_view(Db, ViewDesignId, ViewName, Stale) of
     {ok, View, Group} ->
-        QueryArgs = couch_httpd_view:parse_view_params(Req, Keys, map),
+        QueryArgs = parse_view_params(Req, Keys, map),
         {map, View, Group, QueryArgs};
     {not_found, _Reason} ->
         case couch_view:get_reduce_view(Db, ViewDesignId, ViewName, Stale) of
         {ok, ReduceView, Group} ->
             case Reduce of
             false ->
-                QueryArgs = couch_httpd_view:parse_view_params(Req, Keys, map_red),
+                QueryArgs = parse_view_params(Req, Keys, map_red),
                 MapView = couch_view:extract_map_view(ReduceView),
                 {map, MapView, Group, QueryArgs};
             _ ->
-                QueryArgs = couch_httpd_view:parse_view_params(Req, Keys, reduce),                
+                QueryArgs = parse_view_params(Req, Keys, reduce),
                 {reduce, ReduceView, Group, QueryArgs}
             end;
         {not_found, Reason} ->
@@ -226,8 +236,8 @@ parse_view_params(Req, Keys, ViewType) ->
     QueryList = couch_httpd:qs(Req),
     QueryParams =
     lists:foldl(fun({K, V}, Acc) ->
-            parse_view_param(K, V) ++ Acc
-        end, [], QueryList),
+        parse_view_param(K, V) ++ Acc
+    end, [], QueryList),
     IsMultiGet = (Keys =/= nil),
     Args = #view_query_args{
         view_type=ViewType,
@@ -239,17 +249,17 @@ parse_view_params(Req, Keys, ViewType) ->
 
     GroupLevel = QueryArgs#view_query_args.group_level,
     case {ViewType, GroupLevel, IsMultiGet} of
-        {reduce, exact, true} ->
-            QueryArgs;
-        {reduce, _, false} ->
-            QueryArgs;
-        {reduce, _, _} ->
-            % we can simplify code if we just drop this error message.
-            Msg = <<"Multi-key fetchs for reduce "
-                    "view must include `group=true`">>,
-            throw({query_parse_error, Msg});
-        _ ->
-            QueryArgs
+    {reduce, exact, true} ->
+        QueryArgs;
+    {reduce, _, false} ->
+        QueryArgs;
+    {reduce, _, _} ->
+        % we can simplify code if we just drop this error message.
+        Msg = <<"Multi-key fetchs for reduce "
+                "view must include `group=true`">>,
+        throw({query_parse_error, Msg});
+    _ ->
+        QueryArgs
     end,
     QueryArgs.
 
@@ -302,23 +312,23 @@ parse_view_param(Key, Value) ->
 
 validate_view_query(start_key, Value, Args) ->
     case Args#view_query_args.multi_get of
-        true ->
-            Msg = <<"Query parameter `start_key` is "
-                    "not compatible with multi-get">>,
-            throw({query_parse_error, Msg});
-        _ ->
-            Args#view_query_args{start_key=Value}
+    true ->
+        Msg = <<"Query parameter `start_key` is "
+                "not compatible with multi-get">>,
+        throw({query_parse_error, Msg});
+    _ ->
+        Args#view_query_args{start_key=Value}
     end;
 validate_view_query(start_docid, Value, Args) ->
     Args#view_query_args{start_docid=Value};
 validate_view_query(end_key, Value, Args) ->
     case Args#view_query_args.multi_get of
-        true->
-            Msg = <<"Query parameter `end_key` is "
-                    "not compatible with multi-get">>,
-            throw({query_parse_error, Msg});
-        _ ->
-            Args#view_query_args{end_key=Value}
+    true->
+        Msg = <<"Query parameter `end_key` is "
+                "not compatible with multi-get">>,
+        throw({query_parse_error, Msg});
+    _ ->
+        Args#view_query_args{end_key=Value}
     end;
 validate_view_query(end_docid, Value, Args) ->
     Args#view_query_args{end_docid=Value};
@@ -330,15 +340,15 @@ validate_view_query(stale, _, Args) ->
     Args;
 validate_view_query(descending, true, Args) ->
     case Args#view_query_args.direction of
-        rev -> Args; % Already reversed
-        fwd ->
-            Args#view_query_args{
-                direction = rev,
-                start_docid =
-                    reverse_key_default(Args#view_query_args.start_docid),
-                end_docid =
-                    reverse_key_default(Args#view_query_args.end_docid)
-            }
+    rev -> Args; % Already reversed
+    fwd ->
+        Args#view_query_args{
+            direction = rev,
+            start_docid =
+                reverse_key_default(Args#view_query_args.start_docid),
+            end_docid =
+                reverse_key_default(Args#view_query_args.end_docid)
+        }
     end;
 validate_view_query(descending, false, Args) ->
     Args; % Ignore default condition
@@ -346,38 +356,39 @@ validate_view_query(skip, Value, Args) ->
     Args#view_query_args{skip=Value};
 validate_view_query(group_level, Value, Args) ->
     case Args#view_query_args.view_type of
-        reduce ->
-            Args#view_query_args{group_level=Value};
-        _ ->
-            Msg = <<"Invalid URL parameter 'group' or "
-                    " 'group_level' for non-reduce view.">>,
-            throw({query_parse_error, Msg})
+    reduce ->
+        Args#view_query_args{group_level=Value};
+    _ ->
+        Msg = <<"Invalid URL parameter 'group' or "
+                " 'group_level' for non-reduce view.">>,
+        throw({query_parse_error, Msg})
     end;
 validate_view_query(inclusive_end, Value, Args) ->
     Args#view_query_args{inclusive_end=Value};
 validate_view_query(reduce, _, Args) ->
     case Args#view_query_args.view_type of
-        map ->
-            Msg = <<"Invalid URL parameter `reduce` for map view.">>,
-            throw({query_parse_error, Msg});
-        _ ->
-            Args
+    map ->
+        Msg = <<"Invalid URL parameter `reduce` for map view.">>,
+        throw({query_parse_error, Msg});
+    _ ->
+        Args
     end;
 validate_view_query(include_docs, true, Args) ->
     case Args#view_query_args.view_type of
-        reduce ->
-            Msg = <<"Query parameter `include_docs` "
-                    "is invalid for reduce views.">>,
-            throw({query_parse_error, Msg});
-        _ ->
-            Args#view_query_args{include_docs=true}
+    reduce ->
+        Msg = <<"Query parameter `include_docs` "
+                "is invalid for reduce views.">>,
+        throw({query_parse_error, Msg});
+    _ ->
+        Args#view_query_args{include_docs=true}
     end;
+% Use the view_query_args record's default value
 validate_view_query(include_docs, _Value, Args) ->
     Args;
 validate_view_query(extra, _Value, Args) ->
     Args.
 
-make_view_fold_fun(Req, QueryArgs, Etag, Db, TotalViewCount, HelperFuns) ->
+make_view_fold_fun(Req, QueryArgs, Etag, Db, UpdateSeq, TotalViewCount, HelperFuns) ->
     #view_fold_helper_funs{
         start_response = StartRespFun,
         send_row = SendRowFun,
@@ -388,7 +399,8 @@ make_view_fold_fun(Req, QueryArgs, Etag, Db, TotalViewCount, HelperFuns) ->
         include_docs = IncludeDocs
     } = QueryArgs,
     
-    fun({{Key, DocId}, Value}, OffsetReds, {AccLimit, AccSkip, Resp, RowFunAcc}) ->
+    fun({{Key, DocId}, Value}, OffsetReds,
+            {AccLimit, AccSkip, Resp, RowFunAcc}) ->
         case {AccLimit, AccSkip, Resp} of
         {0, _, _} ->
             % we've done "limit" rows, stop foldling
@@ -400,7 +412,7 @@ make_view_fold_fun(Req, QueryArgs, Etag, Db, TotalViewCount, HelperFuns) ->
             % rendering the first row, first we start the response
             Offset = ReduceCountFun(OffsetReds),
             {ok, Resp2, RowFunAcc0} = StartRespFun(Req, Etag,
-                TotalViewCount, Offset, RowFunAcc),
+                TotalViewCount, Offset, RowFunAcc, UpdateSeq),
             {Go, RowFunAcc2} = SendRowFun(Resp2, Db, {{Key, DocId}, Value},
                 IncludeDocs, RowFunAcc0),
             {Go, {AccLimit - 1, 0, Resp2, RowFunAcc2}};
@@ -412,7 +424,7 @@ make_view_fold_fun(Req, QueryArgs, Etag, Db, TotalViewCount, HelperFuns) ->
         end
     end.
 
-make_reduce_fold_funs(Req, GroupLevel, _QueryArgs, Etag, HelperFuns) ->
+make_reduce_fold_funs(Req, GroupLevel, _QueryArgs, Etag, UpdateSeq, HelperFuns) ->
     #reduce_fold_helper_funs{
         start_response = StartRespFun,
         send_row = SendRowFun
@@ -438,7 +450,7 @@ make_reduce_fold_funs(Req, GroupLevel, _QueryArgs, Etag, HelperFuns) ->
 
     (_Key, Red, {AccLimit, 0, undefined, RowAcc0}) when GroupLevel == 0 ->
         % we haven't started responding yet and group=false
-        {ok, Resp2, RowAcc} = StartRespFun(Req, Etag, RowAcc0),
+        {ok, Resp2, RowAcc} = StartRespFun(Req, Etag, RowAcc0, UpdateSeq),
         {Go, RowAcc2} = SendRowFun(Resp2, {null, Red}, RowAcc),
         {Go, {AccLimit - 1, 0, Resp2, RowAcc2}};
     (_Key, Red, {AccLimit, 0, Resp, RowAcc}) when GroupLevel == 0 ->
@@ -449,18 +461,20 @@ make_reduce_fold_funs(Req, GroupLevel, _QueryArgs, Etag, HelperFuns) ->
     (Key, Red, {AccLimit, 0, undefined, RowAcc0})
             when is_integer(GroupLevel), is_list(Key) ->
         % group_level and we haven't responded yet
-        {ok, Resp2, RowAcc} = StartRespFun(Req, Etag, RowAcc0),
-        {Go, RowAcc2} = SendRowFun(Resp2, {lists:sublist(Key, GroupLevel), Red}, RowAcc),
+        {ok, Resp2, RowAcc} = StartRespFun(Req, Etag, RowAcc0, UpdateSeq),
+        {Go, RowAcc2} = SendRowFun(Resp2,
+                {lists:sublist(Key, GroupLevel), Red}, RowAcc),
         {Go, {AccLimit - 1, 0, Resp2, RowAcc2}};
     (Key, Red, {AccLimit, 0, Resp, RowAcc})
             when is_integer(GroupLevel), is_list(Key) ->
         % group_level and we've already started the response
-        {Go, RowAcc2} = SendRowFun(Resp, {lists:sublist(Key, GroupLevel), Red}, RowAcc),
+        {Go, RowAcc2} = SendRowFun(Resp,
+                {lists:sublist(Key, GroupLevel), Red}, RowAcc),
         {Go, {AccLimit - 1, 0, Resp, RowAcc2}};
 
     (Key, Red, {AccLimit, 0, undefined, RowAcc0}) ->
         % group=true and we haven't responded yet
-        {ok, Resp2, RowAcc} = StartRespFun(Req, Etag, RowAcc0),
+        {ok, Resp2, RowAcc} = StartRespFun(Req, Etag, RowAcc0, UpdateSeq),
         {Go, RowAcc2} = SendRowFun(Resp2, {Key, Red}, RowAcc),
         {Go, {AccLimit - 1, 0, Resp2, RowAcc2}};
     (Key, Red, {AccLimit, 0, Resp, RowAcc}) ->
@@ -470,13 +484,13 @@ make_reduce_fold_funs(Req, GroupLevel, _QueryArgs, Etag, HelperFuns) ->
     end,
     {ok, GroupRowsFun, RespFun}.
 
-apply_default_helper_funs(#view_fold_helper_funs{
-    start_response = StartResp,
-    send_row = SendRow
-}=Helpers) ->
-
+apply_default_helper_funs(
+        #view_fold_helper_funs{
+            start_response = StartResp,
+            send_row = SendRow
+        }=Helpers) ->
     StartResp2 = case StartResp of
-    undefined -> fun json_view_start_resp/5;
+    undefined -> fun json_view_start_resp/6;
     _ -> StartResp
     end,
 
@@ -491,12 +505,13 @@ apply_default_helper_funs(#view_fold_helper_funs{
     };
 
 
-apply_default_helper_funs(#reduce_fold_helper_funs{
-    start_response = StartResp,
-    send_row = SendRow
-}=Helpers) ->
+apply_default_helper_funs(
+        #reduce_fold_helper_funs{
+            start_response = StartResp,
+            send_row = SendRow
+        }=Helpers) ->
     StartResp2 = case StartResp of
-    undefined -> fun json_reduce_start_resp/3;
+    undefined -> fun json_reduce_start_resp/4;
     _ -> StartResp
     end,
 
@@ -511,7 +526,7 @@ apply_default_helper_funs(#reduce_fold_helper_funs{
     }.
 
 make_key_options(#view_query_args{direction = Dir}=QueryArgs) ->
-     [{dir,Dir} | make_start_key_option(QueryArgs) ++ 
+     [{dir,Dir} | make_start_key_option(QueryArgs) ++
             make_end_key_option(QueryArgs)].
 
 make_start_key_option(
@@ -538,10 +553,19 @@ make_end_key_option(
             inclusive_end = false}) ->
     [{end_key_gt, {EndKey,reverse_key_default(EndDocId)}}].
 
-json_view_start_resp(Req, Etag, TotalViewCount, Offset, _Acc) ->
+json_view_start_resp(Req, Etag, TotalViewCount, Offset, _Acc, UpdateSeq) ->
     {ok, Resp} = start_json_response(Req, 200, [{"Etag", Etag}]),
-    BeginBody = io_lib:format("{\"total_rows\":~w,\"offset\":~w,\"rows\":[\r\n",
-            [TotalViewCount, Offset]),
+    BeginBody = case couch_httpd:qs_value(Req, "update_seq") of
+    "true" ->
+        io_lib:format(
+                "{\"total_rows\":~w,\"update_seq\":~w,"
+                "\"offset\":~w,\"rows\":[\r\n",
+                [TotalViewCount, UpdateSeq, Offset]);
+    _Else ->
+        io_lib:format(
+                "{\"total_rows\":~w,\"offset\":~w,\"rows\":[\r\n",
+                [TotalViewCount, Offset])
+    end,
     {ok, Resp, BeginBody}.
 
 send_json_view_row(Resp, Db, {{Key, DocId}, Value}, IncludeDocs, RowFront) ->
@@ -549,9 +573,14 @@ send_json_view_row(Resp, Db, {{Key, DocId}, Value}, IncludeDocs, RowFront) ->
     send_chunk(Resp, RowFront ++  ?JSON_ENCODE(JsonObj)),
     {ok, ",\r\n"}.
 
-json_reduce_start_resp(Req, Etag, _Acc0) ->
+json_reduce_start_resp(Req, Etag, _Acc0, UpdateSeq) ->
     {ok, Resp} = start_json_response(Req, 200, [{"Etag", Etag}]),
-    {ok, Resp, "{\"rows\":[\r\n"}.
+    case couch_httpd:qs_value(Req, "update_seq") of
+    "true" ->
+        {ok, Resp, io_lib:format("{\"update_seq\":~w,\"rows\":[\r\n",[UpdateSeq])};
+    _Else ->
+        {ok, Resp, "{\"rows\":[\r\n"}
+    end.
 
 send_json_reduce_row(Resp, {Key, Value}, RowFront) ->
     send_chunk(Resp, RowFront ++ ?JSON_ENCODE({[{key, Key}, {value, Value}]})),
@@ -573,13 +602,13 @@ view_row_obj(_Db, {{Key, error}, Value}, _IncludeDocs) ->
     {[{key, Key}, {error, Value}]};
 % include docs in the view output
 view_row_obj(Db, {{Key, DocId}, {Props}}, true) ->
-    Rev = case proplists:get_value(<<"_rev">>, Props) of
+    Rev = case couch_util:get_value(<<"_rev">>, Props) of
     undefined ->
         nil;
     Rev0 ->
         couch_doc:parse_rev(Rev0)
     end,
-    IncludeId = proplists:get_value(<<"_id">>, Props, DocId),
+    IncludeId = couch_util:get_value(<<"_id">>, Props, DocId),
     view_row_with_doc(Db, {{Key, DocId}, {Props}}, {IncludeId, Rev});
 view_row_obj(Db, {{Key, DocId}, Value}, true) ->
     view_row_with_doc(Db, {{Key, DocId}, Value}, {DocId, nil});
@@ -593,15 +622,17 @@ view_row_with_doc(Db, {{Key, DocId}, Value}, IdRev) ->
 doc_member(Db, {DocId, Rev}) ->
     ?LOG_DEBUG("Include Doc: ~p ~p", [DocId, Rev]),
     case (catch couch_httpd_db:couch_doc_open(Db, DocId, Rev, [])) of
-        #doc{} = Doc ->
-            JsonDoc = couch_doc:to_json_obj(Doc, []),
-            [{doc, JsonDoc}];
-        _Else ->
-            [{doc, null}]
+    #doc{} = Doc ->
+        JsonDoc = couch_doc:to_json_obj(Doc, []),
+        [{doc, JsonDoc}];
+    _Else ->
+        [{doc, null}]
     end.
-    
 
 finish_view_fold(Req, TotalRows, Offset, FoldResult) ->
+    finish_view_fold(Req, TotalRows, Offset, FoldResult, []).
+
+finish_view_fold(Req, TotalRows, Offset, FoldResult, Fields) ->
     case FoldResult of
     {_, _, undefined, _} ->
         % nothing found in the view or keys, nothing has been returned
@@ -610,7 +641,7 @@ finish_view_fold(Req, TotalRows, Offset, FoldResult) ->
             {total_rows, TotalRows},
             {offset, Offset},
             {rows, []}
-        ]});
+        ] ++ Fields});
     {_, _, Resp, _} ->
         % end the view
         send_chunk(Resp, "\r\n]}"),
@@ -618,11 +649,14 @@ finish_view_fold(Req, TotalRows, Offset, FoldResult) ->
     end.
 
 finish_reduce_fold(Req, Resp) ->
+    finish_reduce_fold(Req, Resp, []).
+
+finish_reduce_fold(Req, Resp, Fields) ->
     case Resp of
     undefined ->
         send_json(Req, 200, {[
             {rows, []}
-        ]});
+        ] ++ Fields});
     Resp ->
         send_chunk(Resp, "\r\n]}"),
         end_json_response(Resp)
@@ -630,11 +664,11 @@ finish_reduce_fold(Req, Resp) ->
 
 parse_bool_param(Val) ->
     case string:to_lower(Val) of
-        "true" -> true;
-        "false" -> false;
-        _ ->
-            Msg = io_lib:format("Invalid boolean parameter: ~p", [Val]),
-            throw({query_parse_error, ?l2b(Msg)})
+    "true" -> true;
+    "false" -> false;
+    _ ->
+        Msg = io_lib:format("Invalid boolean parameter: ~p", [Val]),
+        throw({query_parse_error, ?l2b(Msg)})
     end.
 
 parse_int_param(Val) ->
