@@ -12,7 +12,7 @@
 
 -module(couch_rep).
 -behaviour(gen_server).
--export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, 
+-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
     code_change/3]).
 
 -export([replicate/2, checkpoint/1]).
@@ -68,7 +68,7 @@ replicate({Props}=PostBody, UserCtx) ->
         [?MODULE]
     },
 
-    case proplists:get_value(<<"cancel">>, Props, false) of
+    case couch_util:get_value(<<"cancel">>, Props, false) of
     true ->
  case supervisor:terminate_child(couch_rep_sup, BaseId ++ Extension) of
         {error, not_found} ->
@@ -80,7 +80,7 @@ replicate({Props}=PostBody, UserCtx) ->
     false ->
         Server = start_replication_server(Replicator),
 
-        case proplists:get_value(<<"continuous">>, Props, false) of
+        case couch_util:get_value(<<"continuous">>, Props, false) of
         true ->
             {ok, {continuous, ?l2b(BaseId)}};
         false ->
@@ -111,15 +111,17 @@ init(InitArgs) ->
 do_init([RepId, {PostProps}, UserCtx] = InitArgs) ->
     process_flag(trap_exit, true),
 
-    SourceProps = proplists:get_value(<<"source">>, PostProps),
-    TargetProps = proplists:get_value(<<"target">>, PostProps),
+    SourceProps = couch_util:get_value(<<"source">>, PostProps),
+    TargetProps = couch_util:get_value(<<"target">>, PostProps),
 
-    DocIds = proplists:get_value(<<"doc_ids">>, PostProps, nil),
-    Continuous = proplists:get_value(<<"continuous">>, PostProps, false),
-    CreateTarget = proplists:get_value(<<"create_target">>, PostProps, false),
+    DocIds = couch_util:get_value(<<"doc_ids">>, PostProps, nil),
+    Continuous = couch_util:get_value(<<"continuous">>, PostProps, false),
+    CreateTarget = couch_util:get_value(<<"create_target">>, PostProps, false),
 
-    Source = open_db(SourceProps, UserCtx),
-    Target = open_db(TargetProps, UserCtx, CreateTarget),
+    ProxyParams = parse_proxy_params(
+        couch_util:get_value(<<"proxy">>, PostProps, [])),
+    Source = open_db(SourceProps, UserCtx, ProxyParams),
+    Target = open_db(TargetProps, UserCtx, ProxyParams, CreateTarget),
 
     SourceInfo = dbinfo(Source),
     TargetInfo = dbinfo(Target),
@@ -190,8 +192,8 @@ do_init([RepId, {PostProps}, UserCtx] = InitArgs) ->
         source_log = SourceLog,
         target_log = TargetLog,
         rep_starttime = httpd_util:rfc1123_date(),
-        src_starttime = proplists:get_value(instance_start_time, SourceInfo),
-        tgt_starttime = proplists:get_value(instance_start_time, TargetInfo),
+        src_starttime = couch_util:get_value(instance_start_time, SourceInfo),
+        tgt_starttime = couch_util:get_value(instance_start_time, TargetInfo),
         doc_ids = DocIds
     },
     {ok, State}.
@@ -290,7 +292,9 @@ start_replication_server(Replicator) ->
             {error, {already_started, Pid}} =
                 supervisor:start_child(couch_rep_sup, Replicator),
             ?LOG_DEBUG("replication ~p already running at ~p", [RepId, Pid]),
-            Pid
+            Pid;
+        {error, {db_not_found, DbUrl}} ->
+            throw({db_not_found, <<"could not open ", DbUrl/binary>>})
         end;
     {error, {already_started, Pid}} ->
         ?LOG_DEBUG("replication ~p already running at ~p", [RepId, Pid]),
@@ -302,17 +306,17 @@ start_replication_server(Replicator) ->
 compare_replication_logs(SrcDoc, TgtDoc) ->
     #doc{body={RepRecProps}} = SrcDoc,
     #doc{body={RepRecPropsTgt}} = TgtDoc,
-    case proplists:get_value(<<"session_id">>, RepRecProps) ==
-            proplists:get_value(<<"session_id">>, RepRecPropsTgt) of
+    case couch_util:get_value(<<"session_id">>, RepRecProps) ==
+            couch_util:get_value(<<"session_id">>, RepRecPropsTgt) of
     true ->
         % if the records have the same session id,
         % then we have a valid replication history
-        OldSeqNum = proplists:get_value(<<"source_last_seq">>, RepRecProps, 0),
-        OldHistory = proplists:get_value(<<"history">>, RepRecProps, []),
+        OldSeqNum = couch_util:get_value(<<"source_last_seq">>, RepRecProps, 0),
+        OldHistory = couch_util:get_value(<<"history">>, RepRecProps, []),
         {OldSeqNum, OldHistory};
     false ->
-        SourceHistory = proplists:get_value(<<"history">>, RepRecProps, []),
-        TargetHistory = proplists:get_value(<<"history">>, RepRecPropsTgt, []),
+        SourceHistory = couch_util:get_value(<<"history">>, RepRecProps, []),
+        TargetHistory = couch_util:get_value(<<"history">>, RepRecPropsTgt, []),
         ?LOG_INFO("Replication records differ. "
                 "Scanning histories to find a common ancestor.", []),
         ?LOG_DEBUG("Record on source:~p~nRecord on target:~p~n",
@@ -324,18 +328,18 @@ compare_rep_history(S, T) when S =:= [] orelse T =:= [] ->
     ?LOG_INFO("no common ancestry -- performing full replication", []),
     {0, []};
 compare_rep_history([{S}|SourceRest], [{T}|TargetRest]=Target) ->
-    SourceId = proplists:get_value(<<"session_id">>, S),
+    SourceId = couch_util:get_value(<<"session_id">>, S),
     case has_session_id(SourceId, Target) of
     true ->
-        RecordSeqNum = proplists:get_value(<<"recorded_seq">>, S, 0),
+        RecordSeqNum = couch_util:get_value(<<"recorded_seq">>, S, 0),
         ?LOG_INFO("found a common replication record with source_seq ~p",
             [RecordSeqNum]),
         {RecordSeqNum, SourceRest};
     false ->
-        TargetId = proplists:get_value(<<"session_id">>, T),
+        TargetId = couch_util:get_value(<<"session_id">>, T),
         case has_session_id(TargetId, SourceRest) of
         true ->
-            RecordSeqNum = proplists:get_value(<<"recorded_seq">>, T, 0),
+            RecordSeqNum = couch_util:get_value(<<"recorded_seq">>, T, 0),
             ?LOG_INFO("found a common replication record with source_seq ~p",
                 [RecordSeqNum]),
             {RecordSeqNum, TargetRest};
@@ -350,13 +354,19 @@ close_db(Db) ->
     couch_db:close(Db).
 
 dbname(#http_db{url = Url}) ->
-    Url;
+    strip_password(Url);
 dbname(#db{name = Name}) ->
     Name.
 
+strip_password(Url) ->
+    re:replace(Url,
+        "http(s)?://([^:]+):[^@]+@(.*)$",
+        "http\\1://\\2:*****@\\3",
+        [{return, list}]).
+
 dbinfo(#http_db{} = Db) ->
     {DbProps} = couch_rep_httpc:request(Db),
-    [{list_to_atom(?b2l(K)), V} || {K,V} <- DbProps];
+    [{list_to_existing_atom(?b2l(K)), V} || {K,V} <- DbProps];
 dbinfo(Db) ->
     {ok, Info} = couch_db:get_db_info(Db),
     Info.
@@ -427,7 +437,7 @@ terminate_cleanup(#state{source=Source, target=Target, stats=Stats}) ->
 has_session_id(_SessionId, []) ->
     false;
 has_session_id(SessionId, [{Props} | Rest]) ->
-    case proplists:get_value(<<"session_id">>, Props, nil) of
+    case couch_util:get_value(<<"session_id">>, Props, nil) of
     SessionId ->
         true;
     _Else ->
@@ -436,8 +446,8 @@ has_session_id(SessionId, [{Props} | Rest]) ->
 
 maybe_append_options(Options, Props) ->
     lists:foldl(fun(Option, Acc) ->
-        Acc ++ 
-        case proplists:get_value(Option, Props, false) of
+        Acc ++
+        case couch_util:get_value(Option, Props, false) of
         true ->
             "+" ++ ?b2l(Option);
         false ->
@@ -449,32 +459,32 @@ make_replication_id({Props}, UserCtx) ->
     %% funky algorithm to preserve backwards compatibility
     {ok, HostName} = inet:gethostname(),
     % Port = mochiweb_socket_server:get(couch_httpd, port),
-    Src = get_rep_endpoint(UserCtx, proplists:get_value(<<"source">>, Props)),
-    Tgt = get_rep_endpoint(UserCtx, proplists:get_value(<<"target">>, Props)),    
+    Src = get_rep_endpoint(UserCtx, couch_util:get_value(<<"source">>, Props)),
+    Tgt = get_rep_endpoint(UserCtx, couch_util:get_value(<<"target">>, Props)),
     Base = [HostName, Src, Tgt] ++
-        case proplists:get_value(<<"filter">>, Props) of
+        case couch_util:get_value(<<"filter">>, Props) of
         undefined ->
-            case proplists:get_value(<<"doc_ids">>, Props) of
+            case couch_util:get_value(<<"doc_ids">>, Props) of
             undefined ->
                 [];
             DocIds ->
                 [DocIds]
             end;
         Filter ->
-            [Filter, proplists:get_value(<<"query_params">>, Props, {[]})]
+            [Filter, couch_util:get_value(<<"query_params">>, Props, {[]})]
         end,
     Extension = maybe_append_options(
         [<<"continuous">>, <<"create_target">>], Props),
-    {couch_util:to_hex(erlang:md5(term_to_binary(Base))), Extension}.
+    {couch_util:to_hex(couch_util:md5(term_to_binary(Base))), Extension}.
 
 maybe_add_trailing_slash(Url) ->
     re:replace(Url, "[^/]$", "&/", [{return, list}]).
 
 get_rep_endpoint(_UserCtx, {Props}) ->
-    Url = maybe_add_trailing_slash(proplists:get_value(<<"url">>, Props)),
-    {BinHeaders} = proplists:get_value(<<"headers">>, Props, {[]}),
-    {Auth} = proplists:get_value(<<"auth">>, Props, {[]}),
-    case proplists:get_value(<<"oauth">>, Auth) of
+    Url = maybe_add_trailing_slash(couch_util:get_value(<<"url">>, Props)),
+    {BinHeaders} = couch_util:get_value(<<"headers">>, Props, {[]}),
+    {Auth} = couch_util:get_value(<<"auth">>, Props, {[]}),
+    case couch_util:get_value(<<"oauth">>, Auth) of
     undefined ->
         {remote, Url, [{?b2l(K),?b2l(V)} || {K,V} <- BinHeaders]};
     {OAuth} ->
@@ -509,28 +519,29 @@ open_replication_log(Db, RepId) ->
         #doc{id=DocId}
     end.
 
-open_db(Props, UserCtx) ->
-    open_db(Props, UserCtx, false).
+open_db(Props, UserCtx, ProxyParams) ->
+    open_db(Props, UserCtx, ProxyParams, false).
 
-open_db({Props}, _UserCtx, CreateTarget) ->
-    Url = maybe_add_trailing_slash(proplists:get_value(<<"url">>, Props)),
-    {AuthProps} = proplists:get_value(<<"auth">>, Props, {[]}),
-    {BinHeaders} = proplists:get_value(<<"headers">>, Props, {[]}),
+open_db({Props}, _UserCtx, ProxyParams, CreateTarget) ->
+    Url = maybe_add_trailing_slash(couch_util:get_value(<<"url">>, Props)),
+    {AuthProps} = couch_util:get_value(<<"auth">>, Props, {[]}),
+    {BinHeaders} = couch_util:get_value(<<"headers">>, Props, {[]}),
     Headers = [{?b2l(K),?b2l(V)} || {K,V} <- BinHeaders],
     DefaultHeaders = (#http_db{})#http_db.headers,
-    Db = #http_db{
+    Db1 = #http_db{
         url = Url,
         auth = AuthProps,
         headers = lists:ukeymerge(1, Headers, DefaultHeaders)
     },
+    Db = Db1#http_db{options = Db1#http_db.options ++ ProxyParams},
     couch_rep_httpc:db_exists(Db, CreateTarget);
-open_db(<<"http://",_/binary>>=Url, _, CreateTarget) ->
-    open_db({[{<<"url">>,Url}]}, [], CreateTarget);
-open_db(<<"https://",_/binary>>=Url, _, CreateTarget) ->
-    open_db({[{<<"url">>,Url}]}, [], CreateTarget);
-open_db(<<DbName/binary>>, UserCtx, CreateTarget) ->
+open_db(<<"http://",_/binary>>=Url, _, ProxyParams, CreateTarget) ->
+    open_db({[{<<"url">>,Url}]}, [], ProxyParams, CreateTarget);
+open_db(<<"https://",_/binary>>=Url, _, ProxyParams, CreateTarget) ->
+    open_db({[{<<"url">>,Url}]}, [], ProxyParams, CreateTarget);
+open_db(<<DbName/binary>>, UserCtx, _ProxyParams, CreateTarget) ->
     case CreateTarget of
-    true -> 
+    true ->
         ok = couch_httpd:verify_is_server_admin(UserCtx),
         couch_server:create(DbName, [{user_ctx, UserCtx}]);
     false -> ok
@@ -585,7 +596,7 @@ do_checkpoint(State) ->
             {<<"missing_found">>, ets:lookup_element(Stats, missing_revs, 2)},
             {<<"docs_read">>, ets:lookup_element(Stats, docs_read, 2)},
             {<<"docs_written">>, ets:lookup_element(Stats, docs_written, 2)},
-            {<<"doc_write_failures">>, 
+            {<<"doc_write_failures">>,
                 ets:lookup_element(Stats, doc_write_failures, 2)}
         ]},
         % limit history to 50 entries
@@ -647,15 +658,15 @@ commit_to_both(Source, Target, RequiredSeq) ->
     end,
     {SourceStartTime, TargetStartTime}.
     
-ensure_full_commit(#http_db{} = Target) ->
+ensure_full_commit(#http_db{headers = Headers} = Target) ->
     Req = Target#http_db{
         resource = "_ensure_full_commit",
         method = post,
-        body = true
+        headers = couch_util:proplist_apply_field({"Content-Type", "application/json"}, Headers)
     },
     {ResultProps} = couch_rep_httpc:request(Req),
-    true = proplists:get_value(<<"ok">>, ResultProps),
-    proplists:get_value(<<"instance_start_time">>, ResultProps);
+    true = couch_util:get_value(<<"ok">>, ResultProps),
+    couch_util:get_value(<<"instance_start_time">>, ResultProps);
 ensure_full_commit(Target) ->
     {ok, NewDb} = couch_db:open_int(Target#db.name, []),
     UpdateSeq = couch_db:get_update_seq(Target),
@@ -672,17 +683,17 @@ ensure_full_commit(Target) ->
         InstanceStartTime
     end.
 
-ensure_full_commit(#http_db{} = Source, RequiredSeq) ->
+ensure_full_commit(#http_db{headers = Headers} = Source, RequiredSeq) ->
     Req = Source#http_db{
         resource = "_ensure_full_commit",
         method = post,
-        body = true,
-        qs = [{seq, RequiredSeq}]
+        qs = [{seq, RequiredSeq}],
+        headers = couch_util:proplist_apply_field({"Content-Type", "application/json"}, Headers)
     },
     {ResultProps} = couch_rep_httpc:request(Req),
-    case proplists:get_value(<<"ok">>, ResultProps) of
+    case couch_util:get_value(<<"ok">>, ResultProps) of
     true ->
-        proplists:get_value(<<"instance_start_time">>, ResultProps);
+        couch_util:get_value(<<"instance_start_time">>, ResultProps);
     undefined -> nil end;
 ensure_full_commit(Source, RequiredSeq) ->
     {ok, NewDb} = couch_db:open_int(Source#db.name, []),
@@ -707,7 +718,7 @@ update_local_doc(#http_db{} = Db, #doc{id=DocId} = Doc) ->
         headers = [{"x-couch-full-commit", "false"} | Db#http_db.headers]
     },
     {ResponseMembers} = couch_rep_httpc:request(Req),
-    Rev = proplists:get_value(<<"rev">>, ResponseMembers),
+    Rev = couch_util:get_value(<<"rev">>, ResponseMembers),
     couch_doc:parse_rev(Rev);
 update_local_doc(Db, Doc) ->
     {ok, Result} = couch_db:update_doc(Db, Doc, [delay_commit]),
@@ -720,3 +731,18 @@ up_to_date(Source, Seq) ->
     T = NewDb#db.update_seq == Seq,
     couch_db:close(NewDb),
     T.
+
+parse_proxy_params(ProxyUrl) when is_binary(ProxyUrl) ->
+    parse_proxy_params(?b2l(ProxyUrl));
+parse_proxy_params([]) ->
+    [];
+parse_proxy_params(ProxyUrl) ->
+    {url, _, Base, Port, User, Passwd, _Path, _Proto} =
+        ibrowse_lib:parse_url(ProxyUrl),
+    [{proxy_host, Base}, {proxy_port, Port}] ++
+        case is_list(User) andalso is_list(Passwd) of
+        false ->
+            [];
+        true ->
+            [{proxy_user, User}, {proxy_password, Passwd}]
+        end.
